import { __assign, __rest } from "tslib";
import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import * as React from "react";
import ReachAlert from "@reach/alert";
import { Text } from "./Text";
import { CloseButton } from "./IconButton";
import PropTypes from "prop-types";
import { useTheme } from "./Theme/Providers";
import { IconWrapper } from "./IconWrapper";
import { IconInfo, IconCheckCircle, IconAlertCircle, IconAlertOctagon, IconHelpCircle } from "./Icons";
var alertIntentions = function (theme) { return ({
    info: theme.colors.intent.none,
    success: theme.colors.intent.success,
    question: theme.colors.intent.primary,
    danger: theme.colors.intent.danger,
    warning: theme.colors.intent.warning
}); };
var icons = {
    info: _jsx(IconInfo, {}, void 0),
    success: _jsx(IconCheckCircle, {}, void 0),
    warning: _jsx(IconAlertCircle, {}, void 0),
    danger: _jsx(IconAlertOctagon, {}, void 0),
    question: _jsx(IconHelpCircle, {}, void 0)
};
/**
 * Use an alert to inform users of important information.
 * To display an alert in a toast notification, use the toast module.
 */
export var Alert = function (_a) {
    var children = _a.children, title = _a.title, id = _a.id, subtitle = _a.subtitle, component = _a.component, _b = _a.elevation, elevation = _b === void 0 ? "xs" : _b, onRequestClose = _a.onRequestClose, _c = _a.intent, intent = _c === void 0 ? "info" : _c, other = __rest(_a, ["children", "title", "id", "subtitle", "component", "elevation", "onRequestClose", "intent"]);
    var theme = useTheme();
    var intentions = React.useMemo(function () { return alertIntentions(theme); }, [theme]);
    var dark = theme.colors.mode === "dark";
    var color = intentions[intent];
    var accent = dark ? color.light : color.base;
    if (dark && intent === "info") {
        accent = color.light;
    }
    var icon = icons[intent];
    var contents = title ? (_jsxs("div", __assign({ className: "Alert__title", css: { display: "flex", alignItems: "flex-start" } }, { children: [_jsx("div", __assign({ css: { flex: "0 0 auto", marginTop: "2px" } }, { children: _jsx(IconWrapper, __assign({ size: "md", color: accent }, { children: icon }), void 0) }), void 0),
            _jsxs("div", __assign({ className: "Alert__text-content", css: { marginLeft: theme.spaces.md } }, { children: [_jsx(Text, __assign({ className: "Alert__title-text", id: id, css: { margin: 0 }, variant: "h6" }, { children: title }), void 0),
                    subtitle && (_jsx(Text, __assign({ className: "Alert__title-subtitle", muted: true, css: {
                            fontSize: theme.fontSizes[0]
                        } }, { children: subtitle }), void 0)), children] }), void 0)] }), void 0)) : (children);
    var Component = component || ReachAlert;
    return (_jsx(Component, __assign({ className: "Alert", css: {
            backgroundColor: dark
                ? theme.colors.background.tint1
                : theme.colors.background.default,
            overflow: "hidden",
            position: "relative",
            boxShadow: theme.shadows[elevation],
            borderRadius: theme.radii.md
        } }, other, { children: _jsxs("div", { children: [_jsx("div", { className: "Alert__bar", css: {
                        width: theme.radii.md,
                        position: "absolute",
                        top: 0,
                        left: 0,
                        bottom: 0,
                        backgroundColor: accent
                    } }, void 0),
                _jsxs("div", __assign({ className: "Alert__content", css: {
                        display: "flex",
                        alignItems: "flex-start",
                        padding: theme.spaces.md + " " + theme.spaces.md,
                        paddingRight: onRequestClose ? "3.5rem" : undefined
                    } }, { children: [contents, onRequestClose && (_jsx(CloseButton, { css: {
                                marginTop: "-0.45rem",
                                right: theme.spaces.sm,
                                position: "absolute"
                            }, onClick: onRequestClose }, void 0))] }), void 0)] }, void 0) }), void 0));
};
Alert.displayName = "Alert";
Alert.propTypes = {
    onRequestClose: PropTypes.func,
    subtitle: PropTypes.string,
    title: PropTypes.string,
    id: PropTypes.oneOfType([PropTypes.string]),
    children: PropTypes.node,
    intent: PropTypes.oneOf([
        "info",
        "success",
        "warning",
        "danger",
        "question"
    ]),
    elevation: PropTypes.oneOf(["xs", "sm", "md", "lg", "xl"])
};
//# sourceMappingURL=Alert.js.map