import { __assign, __rest } from "tslib";
import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { css } from "@emotion/react";
import * as React from "react";
import { Layer } from "./Layer";
import PropTypes from "prop-types";
import { useTheme } from "./Theme/Providers";
var TableSectionContext = React.createContext({
    type: "TableHead"
});
var TableContext = React.createContext({ fixed: false });
export var Table = function (_a) {
    var children = _a.children, minWidth = _a.minWidth, fixed = _a.fixed, other = __rest(_a, ["children", "minWidth", "fixed"]);
    var theme = useTheme();
    return (_jsx("div", __assign({ css: {
            width: "100%",
            display: "block",
            overflowX: minWidth ? "auto" : "initial"
        } }, { children: _jsxs("table", __assign({ css: {
                borderSpacing: 0,
                borderCollapse: "separate",
                width: "100%",
                fontFamily: theme.fonts.base,
                WebkitAppearance: "none",
                WebkitFontSmoothing: "antialiased",
                display: "table",
                minWidth: minWidth,
                tableLayout: fixed ? "fixed" : undefined
            } }, other, { children: [fixed && (_jsx("colgroup", { children: fixed.map(function (width, i) {
                        return _jsx("col", { width: width }, i);
                    }) }, void 0)),
                _jsx(TableContext.Provider, __assign({ value: { fixed: fixed ? true : false } }, { children: children }), void 0)] }), void 0) }), void 0));
};
Table.propTypes = {
    minWidth: PropTypes.string,
    fixed: PropTypes.arrayOf(PropTypes.string),
    children: PropTypes.node
};
export function TableHead(_a) {
    var children = _a.children, other = __rest(_a, ["children"]);
    return (_jsx("thead", __assign({ css: {
            display: "table-header-group"
        } }, other, { children: _jsx(TableSectionContext.Provider, __assign({ value: { type: "TableHead" } }, { children: children }), void 0) }), void 0));
}
export var TableRow = function (_a) {
    var onClick = _a.onClick, children = _a.children, other = __rest(_a, ["onClick", "children"]);
    var theme = useTheme();
    var tableSectionType = React.useContext(TableSectionContext).type;
    var buttonProps = onClick
        ? {
            role: "button",
            tabIndex: 0
        }
        : {};
    return (_jsx("tr", __assign({ onClick: onClick, css: {
            height: tableSectionType === "TableHead" ? "31px" : "49px",
            display: "table-row",
            outline: "none",
            verticalAlign: "middle",
            cursor: onClick ? "pointer" : "default",
            ":hover": {
                background: onClick ? theme.colors.background.tint1 : "none"
            }
        } }, buttonProps, other, { children: children }), void 0));
};
TableRow.propTypes = {
    onClick: PropTypes.func,
    children: PropTypes.node
};
/**
 * TableCell, used for both <td> and <th> elements.
 */
var tableCellAlignments = {
    right: css({
        textAlign: "right",
        flexDirection: "row-reverse"
    }),
    left: css({
        textAlign: "left"
    }),
    center: css({
        textAlign: "center"
    }),
    justify: css({
        textAlign: "justify"
    })
};
export var TableCell = function (_a) {
    var _b;
    var _c = _a.align, align = _c === void 0 ? "left" : _c, variant = _a.variant, component = _a.component, ellipsis = _a.ellipsis, children = _a.children, other = __rest(_a, ["align", "variant", "component", "ellipsis", "children"]);
    var theme = useTheme();
    var tableSectionType = React.useContext(TableSectionContext).type;
    var Component = component || (tableSectionType === "TableHead" ? "th" : "td");
    var type = variant || (tableSectionType === "TableHead" ? "head" : "body");
    return (_jsx(Component, __assign({ css: [
            (_b = {
                    zIndex: 4,
                    position: "relative",
                    borderBottom: "1px solid",
                    borderColor: tableSectionType === "TableBody"
                        ? theme.colors.border.muted
                        : theme.colors.border.default,
                    display: "table-cell",
                    padding: theme.spaces.xs + " " + theme.spaces.sm
                },
                _b[theme.mediaQueries.lg] = {
                    paddingLeft: theme.spaces.md,
                    paddingRight: theme.spaces.md
                },
                _b[":last-child"] = {
                    paddingRight: theme.spaces.md
                },
                _b),
            ellipsis && {
                whiteSpace: "nowrap",
                textOverflow: "ellipsis",
                overflow: "hidden"
            },
            type === "head"
                ? {
                    fontWeight: 500,
                    fontSize: theme.fontSizes[0],
                    color: theme.colors.text.muted
                }
                : {
                    fontWeight: 400,
                    fontSize: theme.fontSizes[0],
                    color: theme.colors.text.default
                },
            tableCellAlignments[align]
        ], scope: "col" }, other, { children: children }), void 0));
};
TableCell.propTypes = {
    align: PropTypes.oneOf(["right", "left", "center", "justify"]),
    variant: PropTypes.oneOf(["body", "head"]),
    // component: PropTypes.element,
    ellipsis: PropTypes.bool,
    children: PropTypes.node
};
export function TableBody(_a) {
    var children = _a.children, other = __rest(_a, ["children"]);
    return (_jsx("tbody", __assign({ css: {
            display: "table-row-group"
        } }, other, { children: _jsx(TableSectionContext.Provider, __assign({ value: { type: "TableBody" } }, { children: children }), void 0) }), void 0));
}
export var ExpandingRow = function (_a) {
    var content = _a.content, children = _a.children;
    var theme = useTheme();
    var _b = React.useState(false), selected = _b[0], setSelected = _b[1];
    function close() {
        setSelected(false);
    }
    function open() {
        setSelected(true);
    }
    return (_jsxs(TableBody, { children: [_jsx(TableRow, __assign({ onClick: open }, { children: children }), void 0),
            selected && (_jsx("tr", __assign({ css: { display: "table-row", height: "100px" } }, { children: _jsxs("td", __assign({ colSpan: React.Children.count(children), css: {
                        borderBottom: 0,
                        width: "inherit",
                        padding: 0,
                        position: "relative"
                    } }, { children: [_jsx("div", __assign({ css: {
                                zIndex: 3,
                                width: "inherit",
                                position: "relative",
                                paddingBottom: "24px"
                            } }, { children: _jsx("div", __assign({ css: {
                                    position: "relative",
                                    whiteSpace: "normal",
                                    height: "auto",
                                    display: "block",
                                    paddingTop: "24px"
                                } }, { children: typeof content === "function" ? content(close) : content }), void 0) }), void 0),
                        _jsx(Layer, __assign({ css: {
                                position: "absolute",
                                top: "-49px",
                                left: "-16px",
                                right: "-16px",
                                borderRadius: theme.radii.md,
                                bottom: 0,
                                zIndex: 2
                            } }, { children: null }), void 0)] }), void 0) }), void 0))] }, void 0));
};
ExpandingRow.propTypes = {
    //content: PropTypes.oneOfType([PropTypes.func, PropTypes.node]),
    children: PropTypes.node
};
//# sourceMappingURL=Table.js.map