import { __assign, __rest } from "tslib";
import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { css } from "@emotion/react";
import * as React from "react";
import { Positioner } from "./Positions";
import { Text } from "./Text";
import { animated } from "react-spring";
import PropTypes from "prop-types";
import { isMobile } from "is-mobile";
import { useUid } from "./Hooks/use-uid";
import { useTheme } from "./Theme/Providers";
export var arrowStyles = function (color) {
    return css("\n  position: absolute;\n  width: 3em;\n  height: 3em;\n  &[data-placement*='bottom'] {\n    bottom: 100%;\n    left: 0;\n    margin-top: 0em;\n    width: 1em;\n    height: 0.25em;\n    &::before {\n      width: 10px;\n      border: none;\n      height: 10px;\n      background: " + color + ";\n      border-top: 1px solid " + (color === "white" ? " #dee2e685" : color) + ";\n      border-right: 1px solid " + (color === "white" ? "#dee2e685" : color) + ";\n      transform: rotate(-45deg);\n      border-radius: 2px;\n      margin-top: -1px;\n    }\n  }\n  &[data-placement*='top'] {\n    top: 100%;\n    left: 0;\n    margin-bottom: 0;\n    width: 1em;\n    height: 0.25em;\n    &::before {\n      border-width: 0.25em 0.25em 0 0.25em;\n      border-color: " + color + " transparent transparent transparent;\n    }\n  }\n  &[data-placement*='right'] {\n    right: 100%;\n    height: 1em;\n    width: 0.25em;\n    &::before {\n      border-width: 0.25em 0.25em 0.25em 0;\n      border-color: transparent " + color + " transparent transparent;\n    }\n  }\n  &[data-placement*='left'] {\n    left: 100%;\n    height: 1em;\n    width: 0.25em;\n    &::before {\n      border-width: 0.25em 0 0.25em 0.25em;\n      border-color: transparent transparent transparent " + color + ";\n    }\n  }\n  &::before {\n    content: '';\n    margin: auto;\n    display: block;\n    width: 0;\n    height: 0;\n    border-style: solid;\n  }\n");
};
export var Tooltip = function (_a) {
    var placement = _a.placement, children = _a.children, content = _a.content, _b = _a.delayIn, delayIn = _b === void 0 ? 0 : _b, _c = _a.delayOut, delayOut = _c === void 0 ? 0 : _c, _d = _a.hover, enableHover = _d === void 0 ? true : _d, _e = _a.maxWidth, maxWidth = _e === void 0 ? "300px" : _e, other = __rest(_a, ["placement", "children", "content", "delayIn", "delayOut", "hover", "maxWidth"]);
    var id = useUid();
    var theme = useTheme();
    var dark = theme.colors.mode === "dark";
    var _f = React.useState(false), show = _f[0], setShow = _f[1];
    var _g = React.useState(false), hovering = _g[0], setHovering = _g[1];
    var _h = React.useState(false), delayShow = _h[0], setDelayShow = _h[1];
    function renderTrigger(_a) {
        var ref = _a.ref;
        // We don't want tooltips to show on touch based devices
        // so we just return the child without the event handlers.
        // Really not sure if this is the best way to do this.
        if (isMobile()) {
            return children;
        }
        var child = React.Children.only(children);
        // Enable on non-mobile devices
        return React.cloneElement(child, {
            ref: ref,
            "aria-describedby": id,
            onMouseEnter: function () {
                if (!show)
                    setShow(true);
            },
            onMouseLeave: function () {
                if (show)
                    setShow(false);
            },
            onFocus: function () {
                if (!show)
                    setShow(true);
            },
            onBlur: function () {
                if (show)
                    setShow(false);
            }
        });
    }
    function onMouseLeave() {
        setHovering(false);
    }
    function onMouseEnter() {
        setHovering(true);
    }
    React.useEffect(function () {
        var timeoutIn;
        var timeoutOut;
        if (show) {
            timeoutIn = setTimeout(function () {
                setDelayShow(true);
            }, delayIn);
        }
        else {
            timeoutOut = setTimeout(function () {
                setDelayShow(false);
            }, delayOut);
        }
        return function () {
            if (timeoutIn) {
                clearInterval(timeoutIn);
            }
            if (timeoutOut) {
                clearInterval(timeoutOut);
            }
        };
    }, [delayIn, delayOut, show]);
    return (_jsx(Positioner, __assign({ placement: placement, isOpen: delayShow || (hovering && enableHover), target: renderTrigger }, { children: function (_a, state) {
            var placement = _a.placement, ref = _a.ref, style = _a.style, arrowProps = _a.arrowProps;
            return (_jsxs(animated.div, __assign({ id: id, "data-placement": placement, role: "tooltip", className: "Tooltip", ref: ref, style: __assign(__assign({}, style), { opacity: state.opacity }), css: {
                    zIndex: theme.zIndices.tooltip,
                    margin: theme.spaces.xs
                }, onMouseEnter: onMouseEnter, onMouseLeave: onMouseLeave }, other, { children: [_jsx("div", { className: "Tooltip__content", "data-placement": placement, css: arrowStyles(dark
                            ? theme.colors.background.tint1
                            : theme.colors.palette.gray.dark), ref: arrowProps.ref, style: arrowProps.style }, void 0),
                    _jsx(Text, __assign({ className: "Tooltip__text", variant: "body", css: [
                            {
                                fontSize: theme.fontSizes[0],
                                display: "inline-block",
                                margin: 0,
                                textAlign: "center",
                                maxWidth: maxWidth,
                                boxShadow: theme.shadows.md,
                                borderRadius: theme.radii.sm,
                                padding: theme.spaces.xs + " " + theme.spaces.md,
                                color: "white",
                                background: dark
                                    ? theme.colors.background.tint1
                                    : theme.colors.palette.gray.dark
                            }
                        ] }, { children: content }), void 0)] }), void 0));
        } }), void 0));
};
Tooltip.propTypes = {
    content: PropTypes.node.isRequired,
    children: PropTypes.node.isRequired,
    hover: PropTypes.bool,
    delayIn: PropTypes.number,
    delayOut: PropTypes.number,
    placement: PropTypes.oneOf([
        "auto-start",
        "auto",
        "auto-end",
        "top-start",
        "top",
        "top-end",
        "right-start",
        "right",
        "right-end",
        "bottom-end",
        "bottom",
        "bottom-start",
        "left-end",
        "left",
        "left-start"
    ])
};
//# sourceMappingURL=Tooltip.js.map